@@ -6,9 +6,10 @@ use utf8;
 
 package App::PDF::Link::Icons;
 
+our $VERSION = "0.01";
+
 use Carp;
 use parent qw(Exporter);
-use App::Packager;
 
 our @EXPORT = qw( load_icon_images get_icon );
 
@@ -21,9 +22,6 @@ sub _load_icon_images {
     my %idef =
       ( mscz	    => 'builtin:MuseScore',
 	html	    => 'builtin:iRealPro',
-	sib	    => 'builtin:Sibelius',
-	xml	    => 'builtin:XML',
-	abc	    => 'builtin:ABC',
       );
 
     if ( $env->{all} ) {
